part of ac_dart.host_service.service;


class _AcHostServiceRegistration {
  static const String _consumerInfoPath =
      "plugins/servlet/oauth/consumer-info";
  
  static const String _upmRestPath = "rest/plugins/1.0/";

  final AddonHost _addonHost;

  _AcHostServiceRegistration(AddonHost addonHost)
      : this._addonHost = checkNotNull(addonHost);

  Future installAddonToHost(Uri hostBaseUri, 
                            AcHostHttpClient httpClient) {
    httpClient.addCredentials('Basic', new HttpClientBasicCredentials("admin", "admin"));
    final upmToken = _fetchUpmToken(httpClient);
    
    return upmToken
        .then((Option<String> opt) =>
            opt.map((token) => _sendInstallRequest(_addonHost.baseUrl, httpClient, token)));
  }
  
  Future verifyRegistration(AcHostRegistration hostRegistration, 
                             AcHostHttpClient httpClient) {
    final hostDetails = hostRegistration.hostDetails;
    
    return httpClient.get(_consumerInfoPath).then(
        (HttpClientRequest request) {
      request.headers.add(HttpHeaders.ACCEPT, "application/xml");
      return request.close();
    })
    .then(_asText).then((String str) {
      _log.finer('body is ${str}');
      XmlElement xml = XML.parse(str);
      
      final publicKey = (xml.query('publicKey').single as XmlElement).text;
      final hostKey = (xml.query('key').single as XmlElement).text;
      if (hostDetails.key != hostKey) {
        throw new ArgumentError('provided host key (${hostDetails.key}) does not match that retrieved from host (${hostKey})');
      }
      if (hostDetails.publicKey != publicKey) {
        throw new ArgumentError('provided public key (${hostDetails.publicKey}) does not match that retrieved from host (${publicKey})');
      }
    });

  }

  // the hosts are not setting response headers so darts core helpers treat as binary
  Future<String> _asText(HttpClientResponse resp) {
    return resp
        .transform(new Utf8Decoder())
        .fold(new StringBuffer(), (buffer, data) => buffer..write(data))
        .then((buffer) => buffer.toString());
  }
  
  Future<Option<String>> _fetchUpmToken(AcHostHttpClient httpClient) {
    _log.fine('fetching UPM token');
    return httpClient.open(_upmRestPath, HttpMethod.HEAD)
        .then((HttpClientRequest request) {
          request..followRedirects = false;
          return request.close();
        })
        .then((HttpClientResponse response) {
          final token = response.headers["upm-token"].single;
          _log.fine('UPM token $token');
          return new Some(token);
        })
        .catchError((e) => const None(), test: (e) => e is SocketException);
  }
  
  Future _sendInstallRequest(Uri addonBaseUri, AcHostHttpClient httpClient, String upmToken) {
    return httpClient.post(_upmRestPath, queryParams: {'token': upmToken} )
      .then((HttpClientRequest request) {
          request..followRedirects = false
              ..headers.contentType = new ContentType("application", "vnd.atl.plugins.remote.install+json");
          request.write(JSON.encode({ 'pluginUri': _url.join(addonBaseUri.toString(), 'atlassian-connect.json')}));
          return request.close();
        })
        .then((HttpClientResponse response) {
          final statusCode = response.statusCode;
          final bool success = statusCode >= 200 && statusCode < 300;
          if (statusCode >= 200 && statusCode < 300) {
            _log.info('Successfully sent installation request to ${httpClient.baseUri}.'
                + ' Note Addon is not installed until host registers with us.');
            return new Future.value(true);
          }
          else {
            final msg = "installation failed to ${httpClient.baseUri}. Server returned status: $statusCode";
            _log.warning(msg);
            return new Future.value(false);
//            return new Future.error(msg);
          }
            
        });
    /*
     * .setQueryParameter("token", token)
                        .setHeader("Content-Type", "application/vnd.atl.plugins.remote.install+json")
                        .post("{ \"pluginUri\": \"" + uri + "\" }")
     */
  }

}

